@@ -1,6 +1,6 @@
 package Google::RestApi::DriveApi3::File;
 
-our $VERSION = '1.2.0';
+our $VERSION = '2.0.0';
 
 use Google::RestApi::Setup;
 
@@ -153,8 +153,10 @@ sub permission {
 sub permissions {
   my $self = shift;
   state $check = compile_named(
-    fields => Str, { optional => 1 },
-    params => HashRef, { default => {} },
+    fields        => Str, { optional => 1 },
+    max_pages     => Int, { default => 0 },
+    page_callback => CodeRef, { optional => 1 },
+    params        => HashRef, { default => {} },
   );
   my $p = $check->(@_);
 
@@ -162,16 +164,12 @@ sub permissions {
   $params->{fields} //= 'permissions(id, role, type, emailAddress)';
   $params->{fields} = 'nextPageToken, ' . $params->{fields};
 
-  my @list;
-  my $next_page_token;
-  do {
-    $params->{pageToken} = $next_page_token if $next_page_token;
-    my $result = $self->api(uri => 'permissions', params => $params);
-    push(@list, $result->{permissions}->@*) if $result->{permissions};
-    $next_page_token = $result->{nextPageToken};
-  } until !$next_page_token;
-
-  return @list;
+  return paginate_api(
+    api_call       => sub { $params->{pageToken} = $_[0] if $_[0]; $self->api(uri => 'permissions', params => $params); },
+    result_key     => 'permissions',
+    max_pages      => $p->{max_pages},
+    ($p->{page_callback} ? (page_callback => $p->{page_callback}) : ()),
+  );
 }
 
 sub revision {
@@ -186,8 +184,10 @@ sub revision {
 sub revisions {
   my $self = shift;
   state $check = compile_named(
-    fields => Str, { optional => 1 },
-    params => HashRef, { default => {} },
+    fields        => Str, { optional => 1 },
+    max_pages     => Int, { default => 0 },
+    page_callback => CodeRef, { optional => 1 },
+    params        => HashRef, { default => {} },
   );
   my $p = $check->(@_);
 
@@ -195,16 +195,12 @@ sub revisions {
   $params->{fields} //= 'revisions(id, modifiedTime, keepForever)';
   $params->{fields} = 'nextPageToken, ' . $params->{fields};
 
-  my @list;
-  my $next_page_token;
-  do {
-    $params->{pageToken} = $next_page_token if $next_page_token;
-    my $result = $self->api(uri => 'revisions', params => $params);
-    push(@list, $result->{revisions}->@*) if $result->{revisions};
-    $next_page_token = $result->{nextPageToken};
-  } until !$next_page_token;
-
-  return @list;
+  return paginate_api(
+    api_call       => sub { $params->{pageToken} = $_[0] if $_[0]; $self->api(uri => 'revisions', params => $params); },
+    result_key     => 'revisions',
+    max_pages      => $p->{max_pages},
+    ($p->{page_callback} ? (page_callback => $p->{page_callback}) : ()),
+  );
 }
 
 sub comment {
@@ -221,6 +217,8 @@ sub comments {
   state $check = compile_named(
     fields          => Str, { optional => 1 },
     include_deleted => Bool, { default => 0 },
+    max_pages       => Int, { default => 0 },
+    page_callback   => CodeRef, { optional => 1 },
     params          => HashRef, { default => {} },
   );
   my $p = $check->(@_);
@@ -230,16 +228,12 @@ sub comments {
   $params->{fields} = 'nextPageToken, ' . $params->{fields};
   $params->{includeDeleted} = $p->{include_deleted} ? 'true' : 'false';
 
-  my @list;
-  my $next_page_token;
-  do {
-    $params->{pageToken} = $next_page_token if $next_page_token;
-    my $result = $self->api(uri => 'comments', params => $params);
-    push(@list, $result->{comments}->@*) if $result->{comments};
-    $next_page_token = $result->{nextPageToken};
-  } until !$next_page_token;
-
-  return @list;
+  return paginate_api(
+    api_call       => sub { $params->{pageToken} = $_[0] if $_[0]; $self->api(uri => 'comments', params => $params); },
+    result_key     => 'comments',
+    max_pages      => $p->{max_pages},
+    ($p->{page_callback} ? (page_callback => $p->{page_callback}) : ()),
+  );
 }
 
 sub file_id { shift->{id}; }
@@ -325,25 +319,28 @@ Sets up a notification channel for file changes.
 Returns a Permission object. If id is provided, represents that permission.
 Without id, can be used to create new permissions.
 
-=head2 permissions()
+=head2 permissions(max_pages => $n)
 
-Lists all permissions on the file.
+Lists all permissions on the file. C<max_pages> limits the number of pages
+fetched (default 0 = unlimited).
 
 =head2 revision(id => $id)
 
 Returns a Revision object for the given revision ID.
 
-=head2 revisions()
+=head2 revisions(max_pages => $n)
 
-Lists all revisions of the file.
+Lists all revisions of the file. C<max_pages> limits the number of pages
+fetched (default 0 = unlimited).
 
 =head2 comment(id => $id)
 
 Returns a Comment object. Without id, can be used to create new comments.
 
-=head2 comments(include_deleted => $bool)
+=head2 comments(include_deleted => $bool, max_pages => $n)
 
-Lists all comments on the file.
+Lists all comments on the file. C<max_pages> limits the number of pages
+fetched (default 0 = unlimited).
 
 =head1 AUTHORS
 
